import numpy as np
from sys import path as sys_path
sys_path.append('../')
sys_path.append('../../')
from test_constants import *
import hiperwalk as hpw
import unittest
import networkx as nx
import scipy.sparse

class TestGraph(unittest.TestCase):
    
    def setUp(self):
        self.adj_matrix = [
            [0, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 1, 0, 0, 1, 1, 0, 1, 1],
            [1, 0, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1, 1, 0, 1],
            [0, 0, 0, 1, 0, 0, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 0, 0, 1, 0],
            [0, 0, 1, 0, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 0, 1],
            [1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1],
            [1, 1, 0, 0, 0, 0, 1, 1, 0, 1, 0, 0, 1, 1, 0, 1, 0, 0, 0, 1],
            [1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 0, 0],
            [1, 0, 1, 1, 0, 1, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 1, 1, 0, 0],
            [0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 0, 0],
            [0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 1, 1, 0, 0, 0, 1, 1, 1, 1, 1],
            [0, 0, 1, 0, 1, 0, 0, 0, 1, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 1],
            [0, 0, 1, 0, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1, 0, 0, 1, 1, 0],
            [1, 1, 0, 1, 0, 1, 1, 0, 1, 0, 0, 1, 0, 0, 0, 1, 1, 1, 0, 0],
            [0, 0, 1, 1, 0, 1, 0, 0, 0, 0, 1, 1, 0, 0, 0, 1, 0, 1, 1, 1],
            [0, 1, 1, 0, 0, 0, 1, 1, 1, 0, 0, 1, 0, 0, 0, 0, 1, 1, 0, 1],
            [1, 1, 0, 0, 0, 1, 1, 0, 1, 1, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1],
            [1, 1, 0, 1, 1, 0, 0, 1, 1, 1, 0, 0, 1, 0, 1, 0, 0, 1, 0, 1],
            [0, 1, 0, 0, 1, 0, 0, 1, 0, 1, 1, 1, 1, 1, 1, 0, 1, 0, 1, 1],
            [1, 0, 1, 0, 0, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 1, 0, 0],
            [1, 1, 0, 1, 1, 1, 0, 0, 0, 1, 1, 0, 0, 1, 1, 1, 1, 1, 0, 0]]

    def test_init_with_python_matrix(self):
        g = hpw.Graph(self.adj_matrix)
        adj_matrix = g.adjacency_matrix()
        self.assertTrue(scipy.sparse.issparse(adj_matrix))

        adj_matrix = adj_matrix.todense()
        self.assertTrue(np.all(self.adj_matrix == adj_matrix))

    def test_init_with_numpy_matrix(self):
        numpy_matrix = np.array(self.adj_matrix)
        g = hpw.Graph(numpy_matrix)
        adj_matrix = g.adjacency_matrix()
        self.assertTrue(scipy.sparse.issparse(adj_matrix))

        adj_matrix = adj_matrix.todense()
        self.assertTrue(np.all(numpy_matrix == adj_matrix))

    def test_init_with_csr_array(self):
        sparse_matrix = scipy.sparse.csr_array(self.adj_matrix)
        g = hpw.Graph(sparse_matrix, copy=True)
        adj_matrix = g.adjacency_matrix()
        self.assertTrue(scipy.sparse.issparse(adj_matrix))

        self.assertTrue((sparse_matrix - adj_matrix).nnz == 0)
    
    def test_init_with_networkx_graph(self):
        numpy_matrix = np.array(self.adj_matrix)
        graph = nx.Graph(numpy_matrix)
        g = hpw.Graph(graph)
        adj_matrix = g.adjacency_matrix()
        self.assertTrue(scipy.sparse.issparse(adj_matrix))

        adj_matrix = adj_matrix.todense()
        self.assertTrue(np.all(self.adj_matrix == adj_matrix))

    def test_laplacian_matrix(self):
        D = np.array([
            [10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 13, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 8, 0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 9, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 11, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 6, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12]
        ])
        L = scipy.sparse.csr_array(D - self.adj_matrix) 

        g = hpw.Graph(self.adj_matrix)
        L2 = g.laplacian_matrix()
        self.assertTrue((L - L2).nnz == 0)

        g = hpw.Graph(np.array(self.adj_matrix))
        L2 = g.laplacian_matrix()
        self.assertTrue((L - L2).nnz == 0)

        g = hpw.Graph(np.matrix(self.adj_matrix))
        L2 = g.laplacian_matrix()
        self.assertTrue((L - L2).nnz == 0)

        g = hpw.Graph(scipy.sparse.csr_array(self.adj_matrix))
        L2 = g.laplacian_matrix()
        self.assertTrue((L - L2).nnz == 0)

        g = hpw.Graph(scipy.sparse.csr_matrix(self.adj_matrix))
        L2 = g.laplacian_matrix()
        self.assertTrue((L - L2).nnz == 0)
